# -*- coding: UTF-8 -*-
from abc import abstractmethod

from common import logger


class SessionId(object):
    def __init__(self, shopId, dictionaryId):
        self.shopId = shopId
        self.dictionaryId = dictionaryId

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return NotImplemented

    def __ne__(self, other):
        if isinstance(other, self.__class__):
            return not self.__eq__(other)
        return NotImplemented

    def __hash__(self):
        return hash(tuple(sorted(self.__dict__.items())))


class _SessionService(object):
    sessions = {}
    useSession = False
    sessionIdleTimeout = 300
    singleShopSession = False

    def __init__(self, loggerType):
        self.log = logger.getUploadLogger(loggerType)

    def start_session(self, dictionary_id, node):
        if not self.useSession:
            return {u'success': False, 'info': "Service is not configured for session mode.",
                    'info_ru': u'Сервис не настроен для работы в сессионном режиме'}
        if not self.can_open_session(node.id, dictionary_id):
            info = 'Already opened session for shop %s ' % node.id
            info_ru = u'Уже существует открытая сессия для магазина: %s ' % node.id
            if not self.singleShopSession:
                info += 'and dictionaryId %s' % dictionary_id
                info_ru += u'и справочника %s' % dictionary_id
            self.log.warning(info)
            return {u'success': False, 'info': info, 'info_ru': info_ru}

        self.log.info('Start session %s for dictionaryId %s' % (node.currentConverter.dict_state_id, dictionary_id))
        return self.start_session_worker(node, dictionary_id)

    def can_open_session(self, shop_id, dictionary_id):
        if self.singleShopSession:
            for sessionId in self.sessions.keys():
                if sessionId.shopId == shop_id:
                    return False
            return True
        return not self.is_open_session(shop_id, dictionary_id)

    def is_open_session(self, shop_id, dictionary_id):
        return self.get_session_id(shop_id, dictionary_id) in self.sessions

    @staticmethod
    def get_session_id(shop_id, dictionary_id):
        return SessionId(shop_id, dictionary_id)

    def stop_session(self, shop_code, dictionary_id, shop_id):
        if not self.useSession:
            return {u'success': False, 'info': "Service is not configured for session mode.",
                    'info_ru': u'Сервис не настроен для работы в сессионном режиме'}
        if not self.is_open_session(shop_id, dictionary_id):
            info = 'Session for shopCode %s and dictionaryId %s is not open' % (shop_code, dictionary_id)
            self.log.warning(info)
            return {u'success': False, 'info': info,
                    'info_ru': u'Сессия для магазина с кодом: %s и справочником: %s не открыта' % (
                        shop_code, dictionary_id)}
        node = self.sessions[self.get_session_id(shop_id, dictionary_id)].node
        self.log.info('Stop session %s for shop %s and dictionaryId %s' % (
            node.currentConverter.dict_state_id, shop_code, dictionary_id))
        result = self.stop_session_worker(self.sessions[self.get_session_id(shop_id, dictionary_id)])
        if result:
            del self.sessions[self.get_session_id(shop_id, dictionary_id)]
            return {u'success': True, 'info': "Session for shopCode: %s and dictionary: %s successfully closed" % (
                node.id, dictionary_id),
                    'info_ru': u'Сессия для магазина с кодом: %s и справочником: %s успешно закрыта' % (
                        node.id, dictionary_id)}
        return {u'success': False, 'info': "Error while closing session for shopCode: %s and dictionary: %s" % (
            node.id, dictionary_id),
                'info_ru': u'Произошла ошибка во время закрытия сессии для магазина с кодом: %s и справочником: %s' % (
                    node.id, dictionary_id)}

    def rollback_session(self, shop_code, dictionary_id, session_idle=False, shop_id=None):
        if not self.useSession:
            return {u'success': False, 'info': "Service is not configured for session mode.",
                    'info_ru': u'Сервис не настроен для работы в сессионном режиме'}
        if not self.is_open_session(shop_id, dictionary_id):
            info = 'Session for shopCode %s and dictionaryId %s is not open' % (shop_code, dictionary_id)
            self.log.warning(info)
            return {u'success': False, 'info': info,
                    'info_ru': u'Сессия для магазина с кодом: %s и справочником: %s не открыта' % (
                        shop_code, dictionary_id)}
        node = self.sessions[self.get_session_id(shop_id, dictionary_id)].node
        self.log.info('Rollback session %s for shop %s and dictionaryId %s' % (
            node.currentConverter.dict_state_id, shop_code, dictionary_id))

        result = self.rollback_session_worker(self.sessions[self.get_session_id(shop_id, dictionary_id)], session_idle)
        if result:
            del self.sessions[self.get_session_id(shop_id, dictionary_id)]
            return {u'success': True,
                    'info': "Session for shopCode: %s and dictionaryId: %s successfully rollback and closed" % (
                        shop_code, dictionary_id),
                    'info_ru': u'Сессия для магазина с кодом: %s и справочником: %s успешно отменена' % (
                        shop_code, dictionary_id)}
        else:
            return {u'success': False,
                    'info': "Error while rollback session for shopCode: %s and dictionaryId: %s",
                    'info_ru': u'В процессе отмены сессии для магазина с кодом: %s и справочником: %s произошла ошибка' % (
                        shop_code, dictionary_id)}

    @abstractmethod
    def start_session_worker(self, node, dictionary_id):
        pass

    @abstractmethod
    def stop_session_worker(self, node):
        pass

    @abstractmethod
    def rollback_session_worker(self, node, session_idle):
        pass
